/* devBiTestAsyn.c */
/* base/src/dev devBiTestAsyn.c,v 1.4 2000/02/29 19:35:51 mrk Exp */

/* devBiTestAsyn.c - Device Support Routines for testing asynchronous processing*/
/*
 *      Original Author: Bob Dalesio
 *      Current Author:  Marty Kraimer
 *      Date:            6-1-90
 *
 *      Experimental Physics and Industrial Control System (EPICS)
 *
 *      Copyright 1991, the Regents of the University of California,
 *      and the University of Chicago Board of Governors.
 *
 *      This software was produced under  U.S. Government contracts:
 *      (W-7405-ENG-36) at the Los Alamos National Laboratory,
 *      and (W-31-109-ENG-38) at Argonne National Laboratory.
 *
 *      Initial development by:
 *              The Controls and Automation Group (AT-8)
 *              Ground Test Accelerator
 *              Accelerator Technology Division
 *              Los Alamos National Laboratory
 *
 *      Co-developed with
 *              The Controls and Computing Group
 *              Accelerator Systems Division
 *              Advanced Photon Source
 *              Argonne National Laboratory
 *
 * Modification Log:
 * -----------------
 * .01  11-11-91        jba     Moved set of alarm stat and sevr to macros
 * .02  01-08-92        jba     Added cast in call to wdStart to avoid compile warning msg
 * .03  02-05-92	jba	Changed function arguments from paddr to precord 
 * .04	03-13-92	jba	ANSI C changes
 * .05  04-10-92        jba     pact now used to test for asyn processing, not return value
 * .06  04-05-94        mrk	ANSI changes to callback routines
 *      ...
 */


#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "alarm.h"
#include "callback.h"
#include "dbDefs.h"
#include "dbAccess.h"
#include "recGbl.h"
#include "recSup.h"
#include "devSup.h"
#include "link.h"
#include "dbCommon.h"
#include "biRecord.h"

/* Create the dset for devBiTestAsyn */
static long init_record();
static long read_bi();
struct {
	long		number;
	DEVSUPFUN	report;
	DEVSUPFUN	init;
	DEVSUPFUN	init_record;
	DEVSUPFUN	get_ioint_info;
	DEVSUPFUN	read_bi;
	DEVSUPFUN	special_linconv;
}devBiTestAsyn={
	6,
	NULL,
	NULL,
	init_record,
	NULL,
	read_bi,
	NULL};

static long init_record(pbi)
    struct biRecord	*pbi;
{
    CALLBACK *pcallback;

    /* bi.inp must be a CONSTANT*/
    switch (pbi->inp.type) {
    case (CONSTANT) :
	pcallback = (CALLBACK *)(calloc(1,sizeof(CALLBACK)));
	pbi->dpvt = (void *)pcallback;
	if(recGblInitConstantLink(&pbi->inp,DBF_ENUM,&pbi->val))
	    pbi->udf = FALSE;
	break;
    default :
	recGblRecordError(S_db_badField,(void *)pbi,
		"devBiTestAsyn (init_record) Illegal INP field");
	return(S_db_badField);
    }
    return(0);
}

static long read_bi(pbi)
    struct biRecord	*pbi;
{
    CALLBACK *pcallback=(CALLBACK *)(pbi->dpvt);

    /* bi.inp must be a CONSTANT*/
    switch (pbi->inp.type) {
    case (CONSTANT) :
	if(pbi->pact) {
		printf("%s Completed\n",pbi->name);
		return(2); /* don't convert */
	} else {
                if(pbi->disv<=0) return(2);
                printf("Starting asynchronous processing: %s\n",pbi->name);
                pbi->pact=TRUE;
                callbackRequestProcessCallbackDelayed(
                    pcallback,pbi->prio,pbi,(double)pbi->disv);
		return(0);
	}
    default :
        if(recGblSetSevr(pbi,SOFT_ALARM,INVALID_ALARM)){
		if(pbi->stat!=SOFT_ALARM) {
			recGblRecordError(S_db_badField,(void *)pbi,
			    "devBiTestAsyn (read_bi) Illegal INP field");
		}
	}
    }
    return(0);
}
